%{
#include <stdio.h>
#include "syntax.h"
int yylex();
int yyerror(const char* p) { fprintf(stderr, p); }
%}

%define parse.trace

%union {
       expr_t expr;
       binop_t binop;
       atom_t id;
       num_t num;
       char *sym;
       string_t string;
       object_t object;
}

%token <id> ID
%token <num> NUM
%token <string> STRING

%token STOP
%token INDENT DEDENT
%token IF ELIF ELSE

%right GETS
%left CAT
%left '+' '-'
%left '/' '*'
%left MAPS

%nterm <expr> expr
%nterm <object> objdef

%%
program:	statements | statements statement;
statements:	statements statement STOP
		| statements STOP
		| // null production ;
statement: 	assignment | funcall | conditional;
assignment: 	ID GETS expr;
assignments: 	assignments assignment STOP | // null production;
expr:		funcdef
		| funcall
		| objdef { $$.type = DEE_OBJ; $$.object = $1;}
		| expr CAT expr { $$ = make_binop(CAT, $1, $3); }
		| expr '+' expr { $$ = make_binop('+', $1, $3); }
		| expr '-' expr { $$ = make_binop('-', $1, $3); }
		| expr '*' expr { $$ = make_binop('*', $1, $3); }
		| expr '/' expr { $$ = make_binop('/', $1, $3); }
		| '(' expr ')' {$$ = $2;}
		| ID {$$.id = $1; $$.type = DEE_ID;}
		| NUM {$$.num = $1; $$.type = DEE_NUM;}
		| STRING {$$.string = $1; $$.type = DEE_STR;};
		
funcdef: 	param MAPS expr | param MAPS block;
param:	 	ID | '_' | // null production;

funcall: 	ID '(' exprlist ')'
exprlist:	exprlist ',' expr | expr | // null production;

conditional: 	IF expr block elifs
		| IF expr block elifs ELSE block;
elifs:		ELIF expr block elifs| // null production;

objdef:		'{' block_assignments '}' | '{' '}'

block: 		STOP INDENT statement statements DEDENT
block_assignments: STOP INDENT assignments DEDENT
%%